//************************************************************************
//
// SetupDIMgr.cpp
//
//************************************************************************

#include "StdAfx.h"
#include "SetupDIMgr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


//////////////////////////////////////////////////////////////////////////
// CSetupDIMgr

CSetupDIMgr::CSetupDIMgr(void)
{
    // get a list of devices which support the given interface
    m_devInfo = INVALID_HANDLE_VALUE;
	m_nDevIndex = 0;
	// if we're running under Win2K/NT we might need this...
	m_bUseMultiSZ = (GetVersion() < 0x80000000);
}

CSetupDIMgr::~CSetupDIMgr(void)
{
    ReleaseList();
}

void CSetupDIMgr::ReleaseList(void)
{
    // clean up the device list
    if(INVALID_HANDLE_VALUE != m_devInfo)
    {
        if(!SetupDiDestroyDeviceInfoList(m_devInfo))
        {
            TRACE("unable to delete device info list! error: %u\n",GetLastError());
        }
		m_devInfo = INVALID_HANDLE_VALUE;
    }
}
void CSetupDIMgr::RebuildList(LPCTSTR szBus /* = "PCI" */, BOOL bPresentOnly /* = TRUE */)
{
    ReleaseList();

    // get a list of devices which support the given interface
    m_devInfo = SetupDiGetClassDevs(NULL,
                                   szBus,
                                   0,
                                   DIGCF_ALLCLASSES | (bPresentOnly ? DIGCF_PRESENT : 0));

    if(m_devInfo == INVALID_HANDLE_VALUE )
    {
        TRACE("got INVALID_HANDLE_VALUE!\n");
    }
	m_nDevIndex = 0;
}

BOOL CSetupDIMgr::Enum(void)
{
	if(INVALID_HANDLE_VALUE == m_devInfo)
	{
		return FALSE;
	}

    ZeroMemory(&m_devInfoData, sizeof(m_devInfoData));
    // as per DDK docs on SetupDiEnumDeviceInfo
    m_devInfoData.cbSize = sizeof(m_devInfoData);

    // step through the list of devices for this handle
    // get device info at index deviceIndex, the function returns FALSE
    // when there is no device at the given index.
    if(!SetupDiEnumDeviceInfo(m_devInfo, m_nDevIndex, &m_devInfoData))
	{
		TRACE("Done enumerating...\n");
		return FALSE;
	}

    TRACE("CSetupDIMgr::Enum(): index %d, instance %08x -- ",
        m_nDevIndex, m_devInfoData.DevInst);

	m_nDevIndex++;
	return TRUE;
}

BOOL CSetupDIMgr::GetRegProperty(CString &sProperty, DWORD dwWhich)
{
	if(INVALID_HANDLE_VALUE == m_devInfo)
	{
		return FALSE;
	}

    DWORD regDataType = 0;
	LPTSTR deviceName = (LPTSTR) GetDeviceRegistryProperty(m_devInfo, &m_devInfoData,
		dwWhich, &regDataType );

/*
    TRACE("GetRegProperty(%s, %08x, %08x) -> %s\n", (LPCTSTR)sProperty, m_devInfo, m_devInfoData.DevInst,
        deviceName ? deviceName : "NULL");
*/

	if(NULL != deviceName)
	{
		// just to make sure we are getting the expected type of buffer
		if(REG_SZ != regDataType)
		{
			if(REG_MULTI_SZ == regDataType)
			{
				LPTSTR sScanner = deviceName;
				sProperty.Empty();
				while(0 != _tcslen(sScanner))
				{
					sProperty += sScanner;
					sScanner += _tcslen(sScanner) + sizeof(TCHAR);
					if(0 != _tcslen(sScanner))
                    {
						sProperty += ",";
					}
				}
				free(deviceName);
				return TRUE;
			}

			TRACE("registry key is not an SZ or MULTI_SZ!\n");
			free(deviceName);
			return FALSE;
		}

        sProperty = deviceName;
	    free(deviceName);
		return TRUE;
	}
	return FALSE;
}

BOOL CSetupDIMgr::DeviceName(CString &sDevName)
{
	if(!GetRegProperty(sDevName, SPDRP_FRIENDLYNAME))
		return GetRegProperty(sDevName, SPDRP_DEVICEDESC);
	return TRUE;
}

BOOL CSetupDIMgr::HardwareID(CString &sHardwareID)
{
	return GetRegProperty(sHardwareID, SPDRP_HARDWAREID);
}

BOOL CSetupDIMgr::CompatibleIDs(CString &sCompatibleIDs)
{
    if(!GetRegProperty(sCompatibleIDs, SPDRP_COMPATIBLEIDS))
    {
        return GetRegProperty(sCompatibleIDs, SPDRP_HARDWAREID);
    }
	return TRUE;
}


/*
* A wrapper around SetupDiGetDeviceRegistryProperty, so that I don't have to
* deal with memory allocation anywhere else
*
* parameters:
*   DeviceInfoSet  - The device information set which contains DeviceInfoData
*   DeviceInfoData - Information needed to deal with the given device
*   Property       - which property to get (SPDRP_XXX)
*   PropertyRegDataType - the type of registry property
*/
PBYTE CSetupDIMgr::GetDeviceRegistryProperty(HDEVINFO DeviceInfoSet, 
	PSP_DEVINFO_DATA DeviceInfoData, DWORD Property, PDWORD PropertyRegDataType)
{
	if(INVALID_HANDLE_VALUE == m_devInfo)
	{
		return FALSE;
	}

    DWORD length = 0;
	PBYTE buffer = NULL;

	// get the required length of the buffer
	SetupDiGetDeviceRegistryProperty(DeviceInfoSet, DeviceInfoData, Property, PropertyRegDataType,
										  buffer, length, &length);
	if(0 == length)
	{
		//TRACE("no length...\n");
		return NULL;
	}
	//?
	length += sizeof(TCHAR);

	// since we don't have a buffer yet, it is "insufficient"; we allocate
	// one and try again.
	buffer = (PBYTE) malloc(length);
	if( buffer == NULL )
	{
		TRACE("in GetDeviceRegistryProperty(): "
			   "unable to allocate memory!\n");
		return (NULL);
	}
	if( !SetupDiGetDeviceRegistryProperty( DeviceInfoSet,
										   DeviceInfoData,
										   Property,
										   PropertyRegDataType,
										   buffer,
										   length,
										   NULL// required size
		) )
	{
		TRACE("in GetDeviceRegistryProperty(): "
			   "couldn't get registry property! error: %i\n",
			   GetLastError());
		free( buffer );
		return (NULL);
	}

    if(length != sizeof(TCHAR) * (_tcslen((LPCTSTR)buffer)+2))
    {
        // must be multi-sz, even though it's not declared as such!
        // (Win98 does that)
        *PropertyRegDataType = REG_MULTI_SZ;
    }

	// ok, we are finally done, and can return the buffer
	return (buffer);
}

//** end of SetupDIMgr.cpp ***********************************************
